@@ -59,7 +59,7 @@ typedef struct _CadPulseOperation {
 
 static void pulseaudio_cleanup(CadPulse *self);
 static gboolean pulseaudio_connect(CadPulse *self);
-static void init_pulseaudio_objects(CadPulse *self);
+static gboolean init_pulseaudio_objects(CadPulse *self);
 
 /******************************************************************************
  * Source management
@@ -149,6 +149,8 @@ static void process_new_source(CadPulse *self, const pa_source_info *info)
     prop = pa_proplist_gets(info->proplist, PA_PROP_DEVICE_CLASS);
     if (prop && strcmp(prop, SINK_CLASS) != 0)
         return;
+    if (g_str_has_suffix(info->name, "monitor"))
+        return;
     if (info->card != self->card_id || self->source_id != -1)
         return;
 
@@ -185,9 +187,13 @@ static void init_source_info(pa_context *ctx, const pa_source_info *info, int eo
     }
 
     process_new_source(self, info);
-    if (self->source_id < 0)
+    if (self->source_id < 0 || self->source_id != info->index)
         return;
 
+    op = pa_context_set_default_source(ctx, info->name, NULL, NULL);
+    if (op)
+        pa_operation_unref(op);
+
     target_port = get_available_source_port(info, NULL);
     if (target_port) {
         op = pa_context_set_source_port_by_index(ctx, self->source_id,
@@ -334,9 +340,13 @@ static void init_sink_info(pa_context *ctx, const pa_sink_info *info, int eol, v
     }
 
     process_new_sink(self, info);
-    if (self->sink_id < 0)
+    if (self->sink_id < 0 || self->sink_id != info->index)
         return;
 
+    op = pa_context_set_default_sink(ctx, info->name, NULL, NULL);
+    if (op)
+        pa_operation_unref(op);
+
     target_port = get_available_sink_port(info, NULL);
     if (target_port) {
         g_debug("  Using sink port '%s'", target_port);
@@ -394,7 +404,8 @@ static void init_card_info(pa_context *ctx, const pa_card_info *info, int eol, v
 
         if (strstr(port->name, SND_USE_CASE_DEV_SPEAKER) != NULL) {
             has_speaker = TRUE;
-        } else if (strstr(port->name, SND_USE_CASE_DEV_EARPIECE) != NULL) {
+        } else if (strstr(port->name, SND_USE_CASE_DEV_EARPIECE) != NULL ||
+                   strstr(port->name, SND_USE_CASE_DEV_HANDSET)  != NULL) {
             has_earpiece = TRUE;
         }
     }
@@ -436,42 +447,44 @@ static void init_card_info(pa_context *ctx, const pa_card_info *info, int eol, v
  * state of PulseAudio objects
  ******************************************************************************/
 
- static void init_module_info(pa_context *ctx, const pa_module_info *info, int eol, void *data)
- {
-     pa_operation *op;
+static void init_module_info(pa_context *ctx, const pa_module_info *info, int eol, void *data)
+{
+    pa_operation *op;
 
-     if (eol != 0)
-         return;
+    if (eol != 0)
+        return;
 
-     if (!info) {
-         g_critical("PA returned no module info (eol=%d)", eol);
-         return;
-     }
+    if (!info) {
+        g_critical("PA returned no module info (eol=%d)", eol);
+        return;
+    }
 
-     g_debug("MODULE: idx=%u name='%s'", info->index, info->name);
+    g_debug("MODULE: idx=%u name='%s'", info->index, info->name);
 
-     if (strcmp(info->name, "module-switch-on-port-available") == 0) {
-         g_debug("MODULE: unloading '%s'", info->name);
-         op = pa_context_unload_module(ctx, info->index, NULL, NULL);
-         if (op)
-             pa_operation_unref(op);
-     }
- }
+    if (strcmp(info->name, "module-switch-on-port-available") == 0) {
+        g_debug("MODULE: unloading '%s'", info->name);
+        op = pa_context_unload_module(ctx, info->index, NULL, NULL);
+        if (op)
+            pa_operation_unref(op);
+    }
+}
 
- static void init_pulseaudio_objects(CadPulse *self)
- {
-     pa_operation *op;
+static gboolean init_pulseaudio_objects(CadPulse *self)
+{
+    pa_operation *op;
 
-     self->card_id = self->sink_id = self->source_id = -1;
-     self->sink_ports = self->source_ports = NULL;
+    self->card_id = self->sink_id = self->source_id = -1;
+    self->sink_ports = self->source_ports = NULL;
 
-     op = pa_context_get_card_info_list(self->ctx, init_card_info, self);
-     if (op)
-         pa_operation_unref(op);
-     op = pa_context_get_module_info_list(self->ctx, init_module_info, self);
-     if (op)
-         pa_operation_unref(op);
- }
+    op = pa_context_get_card_info_list(self->ctx, init_card_info, self);
+    if (op)
+        pa_operation_unref(op);
+    op = pa_context_get_module_info_list(self->ctx, init_module_info, self);
+    if (op)
+        pa_operation_unref(op);
+
+    return G_SOURCE_REMOVE;
+}
 
 static void changed_cb(pa_context *ctx, pa_subscription_event_type_t type, uint32_t idx, void *data)
 {
@@ -500,7 +513,7 @@ static void changed_cb(pa_context *ctx, pa_subscription_event_type_t type, uint3
             g_hash_table_destroy(self->source_ports);
             self->source_ports = NULL;
         } else if (kind == PA_SUBSCRIPTION_EVENT_NEW) {
-            g_debug("new sink %u", idx);
+            g_debug("new source %u", idx);
             op = pa_context_get_source_info_by_index(ctx, idx, init_source_info, self);
             if (op)
                 pa_operation_unref(op);
